package com.rv.im;

import android.app.Service;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Binder;
import android.os.IBinder;
import android.text.TextUtils;
import android.util.Log;

import com.rv.im.broadcast.MessageReceiver;
import com.rv.im.db.table.ChatMessage;
import com.rv.im.enums.CallTypeEnum;
import com.rv.im.listener.AbstractMsgListener;
import com.rv.im.log.LogUtil;
import com.rv.im.util.NetworkUtil;
import com.rv.im.xmpp.ListenerManager;
import com.rv.im.xmpp.NotifyConnectionListener;
import com.rv.im.xmpp.ReceiptManager;
import com.rv.im.xmpp.XChatManager;
import com.rv.im.xmpp.XmppConnectionManager;
import com.rv.im.xmpp.listener.AuthStateListener;
import com.rv.im.xmpp.listener.ChatMessageListener;

import org.greenrobot.eventbus.EventBus;
import org.jivesoftware.smack.XMPPConnection;

import java.util.UUID;

public class ImService extends Service {

    private static final String TAG = ImService.class.getSimpleName();
    private static final Intent SERVICE_INTENT = new Intent();
    private XmppConnectionManager mConnectionManager;// 唯一
    private boolean isInit = true;
    private String mLoginUserId;
    private String mLoginPassword;
    private String mLoginNickName;

    private XChatManager mXChatManager;// 唯一
    private ReceiptManager mReceiptManager;// 唯一

    public String getUserId() {
        return mLoginUserId;
    }

    public static Intent getIntent() {
        return SERVICE_INTENT;
    }

    private MessageReceiver receiver = new MessageReceiver(new AbstractMsgListener() {
        @Override
        public void send(String toUserId, String message) {
            ChatMessage chatMessage = ChatMessage.json2Message(message);
            sendChatMessage(toUserId, chatMessage);
        }
    });

    private NotifyConnectionListener mNotifyConnectionListener = new NotifyConnectionListener() {
        @Override
        public void notifyConnecting() {
            LogUtil.e(TAG, "xmpp:notifyConnecting");
            ListenerManager.getInstance().notifyAuthStateChange(AuthStateListener.AUTH_STATE_ING);
        }

        @Override
        public void notifyConnected(XMPPConnection arg0) {
            LogUtil.e(TAG, "xmpp:notifyConnected");
            ListenerManager.getInstance().notifyAuthStateChange(AuthStateListener.AUTH_STATE_ING);
        }

        @Override
        public void notifyAuthenticated(XMPPConnection arg0) {
            LogUtil.e(TAG, "xmpp:notifyAuthenticated");
            onAuthenticated();
            ListenerManager.getInstance().notifyAuthStateChange(AuthStateListener.AUTH_STATE_SUCCESS);// 通知登陆成功
            getLastChatHistory();
        }

        @Override
        public void notifyConnectionClosedOnError(Exception e) {

            ListenerManager.getInstance().notifyAuthStateChange(AuthStateListener.AUTH_STATE_NOT);
            LogUtil.e(TAG, "xmpp:notifyConnectionClosedOnError");
            Log.e(TAG, "连接异常==" + e.getMessage());
            if (e.getMessage().contains("conflict")) {
                Log.e(TAG, "连接异常==被挤下线了");
            } else {
                init();
            }
        }

        @Override
        public void notifyConnectionClosed() {
            LogUtil.e(TAG, "xmpp:notifyConnectionClosed");
            ListenerManager.getInstance().notifyAuthStateChange(AuthStateListener.AUTH_STATE_NOT);
        }
    };

    private ImServiceBinder mBinder = new ImServiceBinder();

    @Override
    public IBinder onBind(Intent intent) {
        LogUtil.d(TAG, "ImService onBind");
        return mBinder;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        registerReceiver();

        mLoginUserId = ImSetting.getUserId();
        mLoginPassword = ImSetting.getUserPassword();
        mLoginNickName = ImSetting.getUserName();
        init();
    }

    private void init() {
        if (mConnectionManager == null) {
            initConnection();
        }
        Log.e("zq", "isInit==true,直接登录");
        login(mLoginUserId, mLoginPassword);
        return;
    }

    private void onAuthenticated() {
        if (!isAuthenticated()) {
            return;
        }

        /* 消息回执管理 */
        if (mReceiptManager == null) {
            mReceiptManager = new ReceiptManager(this, mConnectionManager.getConnection());
        } else {
            mReceiptManager.reset();
        }

        // 初始化消息处理
        if (mXChatManager == null) {
            mXChatManager = new XChatManager(this, mConnectionManager.getConnection());
        } else {
            mXChatManager.reset();
        }

        /*  获取离线消息 */
        mConnectionManager.sendOnLineMessage();
    }

    public boolean isAuthenticated() {
        if (mConnectionManager != null && mConnectionManager.isAuthenticated()) {
            return true;
        }
        return false;
    }

    public void initConnection() {
        mConnectionManager = new XmppConnectionManager(this, mNotifyConnectionListener);
    }

    public void login(String userId, String password) {
        if (!TextUtils.isEmpty(userId) && !TextUtils.isEmpty(password) && mConnectionManager != null) {
            mConnectionManager.login(userId, password);
        }
    }


    public void getLastChatHistory() {
//        Map<String, String> params = new HashMap();
//        params.put("access_token", ImSetting.getImToken());
//        long syncTimeLen = PreferenceUtils.getLong(ImSetting.getContext(), Constants.OFFLINE_TIME + mLoginUserId, 0);
//        params.put("startTime", String.valueOf(syncTimeLen));
//
//
//        OkGoUtil
//
//        AppConfig appConfig = (AppConfig) RvCache.getInstance(getApplicationContext()).get(CacheEnum.IM_CONFIG);
//        OkGoUtil.postJson(appConfig.GET_LAST_CHAT_LIST, params, new StringCallback() {
//            @Override
//            public void onSuccess(Response<String> response) {
//                String string = response.body();
//
//            }
//        });
//        new.url(CoreManager.requireConfig(this).GET_LAST_CHAT_LIST)
//                .params(params)
//                .build()
//                .execute(new ListCallback<LastChatHistoryList>(LastChatHistoryList.class) {
//                    @Override
//                    public void onResponse(ArrayResult<LastChatHistoryList> result) {
//                        if (result.getResultCode() == 1 && result.getData() != null) {
//                            final List<LastChatHistoryList> data = result.getData();
//                            new Thread(new Runnable() {
//                                @Override
//                                public void run() {
//                                    for (int i = 0; i < data.size(); i++) {
//                                        LastChatHistoryList mLastChatHistoryList = data.get(i);
//
//                                        if (mLastChatHistoryList.getIsRoom() == 1) {// 群组消息
//                                            // 取出该群组最后一条消息
//                                            ChatMessage mLocalLastMessage = ChatMessageDao.getInstance().getLastChatMessage(mLoginUserId, mLastChatHistoryList.getJid());
//                                            if (mLocalLastMessage == null
//                                                    || mLocalLastMessage.getPacketId().equals(mLastChatHistoryList.getMessageId())) {
//                                                // 最后一条消息为空(代表本地为空)
//                                                // || 最后一条消息的msgId==服务端记录的该群组最后一条消息msgId(代表离线期间无消息产生 理论上服务端这个情况服务端是不会返回的) 不需要生成任务
//                                            } else {
//                                                // 生成一条群组漫游任务，存入任务表
//                                                MsgRoamTask mMsgRoamTask = new MsgRoamTask();
//                                                mMsgRoamTask.setTaskId(System.currentTimeMillis());
//                                                mMsgRoamTask.setOwnerId(mLoginUserId);
//                                                mMsgRoamTask.setUserId(mLastChatHistoryList.getJid());
//                                                mMsgRoamTask.setStartTime(mLocalLastMessage.getTimeSend());
//                                                mMsgRoamTask.setStartMsgId(mLocalLastMessage.getPacketId());
//                                                MsgRoamTaskDao.getInstance().createMsgRoamTask(mMsgRoamTask);
//                                            }
//                                        }
//                                        // 更新朋友表部分字段，用于显示
//                                        String str;
//                                        if (mLastChatHistoryList.getIsEncrypt() == 1) {// 需要解密
//                                            String content = mLastChatHistoryList.getContent().replaceAll("\n", "");
//                                            String decryptKey = Md5Util.toMD5(AppConfig.apiKey + mLastChatHistoryList.getTimeSend() + mLastChatHistoryList.getMessageId());
//                                            try {
//                                                str = DES.decryptDES(content, decryptKey);
//                                            } catch (Exception e) {
//                                                str = mLastChatHistoryList.getContent();
//                                                e.printStackTrace();
//                                            }
//                                        } else {
//                                            str = mLastChatHistoryList.getContent();
//                                        }
//
//                                        FriendDao.getInstance().updateApartDownloadTime(mLastChatHistoryList.getUserId(), mLastChatHistoryList.getJid(),
//                                                str, mLastChatHistoryList.getType(), mLastChatHistoryList.getTimeSend(),
//                                                mLastChatHistoryList.getIsRoom(), mLastChatHistoryList.getFrom(), mLastChatHistoryList.getFromUserName(),
//                                                mLastChatHistoryList.getToUserName());
//                                    }
//                                    MsgBroadcast.broadcastMsgUiUpdate(MyApplication.getContext());
//                                    // 以上任务生成之后，在通知XMPP加入群组 获取群组离线消息
//                                    joinExistGroup();
//                                }
//                            }).start();
//                        } else {// 数据异常，也需要调用XMPP加入群组
//                            joinExistGroup();
//                        }
//                    }
//
//                    @Override
//                    public void onError(Call call, Exception e) {
//                        // 同上
//                        joinExistGroup();
//                    }
//                });
    }


//    // 发送上线、检测消息
//    public void sendOnLineMessage() {
//        ChatMessage chatMessage = new ChatMessage();
//        chatMessage.setMessageType(XmppMessage.TYPE_SEND_ONLINE_STATUS);
//
//        chatMessage.setFromUserId(mLoginUserId);
//        chatMessage.setFromUserName(mLoginNickName);
//        chatMessage.setToUserId(mLoginUserId);
//        chatMessage.setContent("1");// 0 离线 1 在线
//
//        chatMessage.setPacketId(UUID.randomUUID().toString().replaceAll("-", ""));
//        sendChatMessage(mLoginUserId, chatMessage);
//    }


    /********
     * 发送忙线消息
     * @param messageType
     * @param toUserId
     */
    public void sendBusyMessage(int messageType, String toUserId) {

        ChatMessage chatMessage = new ChatMessage();
        chatMessage.setMessageType(messageType);
        chatMessage.setCallType(CallTypeEnum.CALL_BUSY);
        chatMessage.setFromUserId(mLoginUserId);
        chatMessage.setFromUserName(mLoginNickName);
        chatMessage.setToUserId(toUserId);
        chatMessage.setContent("对方忙线");
        chatMessage.setTimeSend(System.currentTimeMillis());
        chatMessage.setPackId(UUID.randomUUID().toString().replaceAll("-", ""));
        sendChatMessage(toUserId, chatMessage);
    }


    public class ImServiceBinder extends Binder {
        public ImService getService() {
            return ImService.this;
        }
    }

    /******
     * 注册接受消息的广播
     */
    private void registerReceiver() {
        IntentFilter intentFilter = new IntentFilter();
        intentFilter.addAction(Constants.ACTIONS.ACTION_SEND_RECEIVER);
        registerReceiver(receiver, intentFilter);
    }

    private void release() {
        if (mConnectionManager != null) {
            mConnectionManager.release();
            mConnectionManager = null;
        }

        mReceiptManager = null;
        mXChatManager = null;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        release();

        if (receiver != null) {
            unregisterReceiver(receiver);
            receiver = null;
        }
    }

    public void logout() {
        isInit = false;
        if (mConnectionManager != null) {
            mConnectionManager.logout();
        }
        stopSelf();
    }

    /**
     * 发送聊天消息
     */
    public void sendChatMessage(String toUserId, ChatMessage chatMessage) {
        if (mXChatManager == null) {
            LogUtil.e(TAG, "mXChatManager==null");
        }

        if (mReceiptManager == null) {
            LogUtil.e(TAG, "mReceiptManager==null");
        }

        if (!isAuthenticated()) {
            Log.e(TAG, "isAuthenticated==false");
            ListenerManager.getInstance().notifyMessageSendStateChange(chatMessage.getPackId(), ChatMessageListener.MESSAGE_SEND_FAILED);
            return;
        }

        if (mXChatManager == null || mReceiptManager == null
                || (!isAuthenticated() && !NetworkUtil.isNetworkAvailable(getApplicationContext()))) {
            ListenerManager.getInstance().notifyMessageSendStateChange(chatMessage.getPackId(), ChatMessageListener.MESSAGE_SEND_FAILED);
        } else {
            /**
             * 先添加一个等待接收回执的消息
             * 然后再发送这条消息
             */
            mReceiptManager.addWillSendMessage(toUserId, chatMessage, ReceiptManager.SendType.NORMAL, chatMessage.getContent());
            mXChatManager.sendMessage(toUserId, chatMessage);
        }
    }
}
